unit minedoutspielfeld;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, minedoutbase;

type
 { TSpielfeld }

  TSpielfeld = class
    private
      FSpielfeld : TSpielfeldArray;
      FSpielerposition : TPos;
      function getSpielerposition : TPos;
      procedure BewegeSpieler(Richtung : TRichtung);  // Wirft TAktionBewegung

    public
      BeiSpielerBewegung : TAktionBewegung;
      constructor Create;
      destructor Destroy; override;
      procedure LegeMinen(AnzahlMinen : Integer);
      procedure VerarbeiteUserAktion(const DieAktion : TAktionen); // für UserAktion
      property Spielerposition : TPos read getSpielerposition;
      { TODO : MinenInDerNaehe noch austesten! }
      function MinenInDerNaehe(Punkt : TPos) : Integer;
      function getInhalt(Punkt: TPos): TFeldinhalt;
      function getAlleInhalte : TSpielfeldArray;

  end;



implementation

{ TSpielfeld }

constructor TSpielfeld.Create;
var
  i, j : Integer;
begin
  (* FSpielfeld als Leer initialisieren *)
  for i:= WESTRAND to OSTRAND do
    for j:= SUEDRAND to NORDRAND do
      FSpielfeld[i,j] := TFeldinhalt.Leer;

  (* Starposition FSpieler festlegen *)
  FSpielerposition.X:= OSTRAND Div 2;
  FSpielerposition.Y:= SUEDRAND;
  FSpielfeld[FSpielerposition.X, FSpielerposition.Y] := TFeldinhalt.Spieler;

  (* FZiel festlegen *)
  FSpielfeld[OSTRAND Div 2, NORDRAND] := TFeldinhalt.Ziel;

  end;

destructor TSpielfeld.Destroy;
begin
  inherited Destroy;
end;


procedure TSpielfeld.LegeMinen(AnzahlMinen : Integer);
var
  y: Integer;
  x: Integer;
begin
  (* Minen legen *)
    Randomize;
    while AnzahlMinen > 0 do
      begin
        x := Random(OSTRAND);
        y := Random(NORDRAND);
        (* Minen dürfen nicht auf der ersten und letzten Linie liegen! *)
        if (y > SUEDRAND) and (y < NORDRAND) then
          if FSpielfeld[x, y] = Leer then
            begin
              FSpielfeld[x, y] := Mine;
              Dec(AnzahlMinen);
            end;
        end;
end;

procedure TSpielfeld.VerarbeiteUserAktion(const DieAktion: TAktionen);
begin
  Case DieAktion of
    TAktionen.uaNorden:
      BewegeSpieler(TRichtung.rNorden);
    TAktionen.uaSueden:
      BewegeSpieler(TRichtung.rSueden);
    TAktionen.uaOsten:
      BewegeSpieler(TRichtung.rOsten);
    TAktionen.uaWesten:
      BewegeSpieler(TRichtung.rWesten);
  end; // andere Aktionen werden nicht vom Spielfeld weiterverwendet!
end;

function TSpielfeld.MinenInDerNaehe(Punkt : TPos) : Integer;
begin
  Result := 0;
  // NORDRAND
  if Punkt.Y < NORDRAND then
    if FSpielfeld[Punkt.X, Punkt.Y+1] = TFeldinhalt.Mine then
      Inc(Result);
  // Süden
  if Punkt.Y > SUEDRAND then
    if FSpielfeld[Punkt.X, Punkt.Y-1] = TFeldinhalt.Mine then
      Inc(Result);
  // OSTRAND
  if Punkt.X < OSTRAND then
    if FSpielfeld[Punkt.X+1, Punkt.Y] = TFeldinhalt.Mine then
      Inc(Result);
  // WESTRAND
  if Punkt.X > WESTRAND then
    if FSpielfeld[Punkt.X-1, Punkt.Y] = TFeldinhalt.Mine then
      Inc(Result);

end;

function TSpielfeld.getSpielerposition: TPos;
begin
  Result := FSpielerposition;
end;

procedure TSpielfeld.BewegeSpieler(Richtung: TRichtung);
var
  FBewegungErfolgreich : Boolean;
begin
  FBewegungErfolgreich:= False;
  Case Richtung of
    TRichtung.rNorden:
      begin
        if FSpielerposition.Y < NORDRAND then
          begin
            FSpielfeld[FSpielerposition.X, FSpielerposition.Y]
                                           := TFeldinhalt.Leer;
            Inc(FSpielerposition.Y);
            FSpielfeld[FSpielerposition.X, FSpielerposition.Y]
                                           := TFeldinhalt.Spieler;
            FBewegungErfolgreich:=True;
          end;
      end;
    TRichtung.rSueden:
      begin
        if FSpielerposition.Y > SUEDRAND then
          begin
            FSpielfeld[FSpielerposition.X, FSpielerposition.Y]
                                           := TFeldinhalt.Leer;
            Dec(FSpielerposition.Y);
            FSpielfeld[FSpielerposition.X, FSpielerposition.Y]
                                           := TFeldinhalt.Spieler;
            FBewegungErfolgreich:= True;
          end;
      end;
    TRichtung.rOsten:
      begin
        if FSpielerposition.X < OSTRAND then
          begin
            FSpielfeld[FSpielerposition.X, FSpielerposition.Y]
                                           := TFeldinhalt.Leer;
            Inc(FSpielerposition.X);
            FSpielfeld[FSpielerposition.X, FSpielerposition.Y]
                                           := TFeldinhalt.Spieler;
            FBewegungErfolgreich:=True;
          end;
      end;
    TRichtung.rWesten:
      begin
        if FSpielerposition.X > WESTRAND then
          begin
            FSpielfeld[FSpielerposition.X, FSpielerposition.Y]
                                           := TFeldinhalt.Leer;
            Dec(FSpielerposition.X);
            FSpielfeld[FSpielerposition.X, FSpielerposition.Y]
                                           := TFeldinhalt.Spieler;
            FBewegungErfolgreich:=True;
          end;
      end;
  end;
  if FBewegungErfolgreich = True then
    if Assigned(BeiSpielerBewegung) then
      BeiSpielerBewegung(FSpielerposition);
end;

function TSpielfeld.getInhalt(Punkt: TPos): TFeldinhalt;
begin
  Result := FSpielfeld[Punkt.X, Punkt.Y];
end;

function TSpielfeld.getAlleInhalte: TSpielfeldArray;
begin
  Result := FSpielfeld;
end;





end.

